/* ee-routines for Chicken Mobile Home */

#ifndef cmh_f_cpu_
#define cmh_f_cpu_

#define F_CPU 8000000L

#endif

#include <inttypes.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include "chicken_mobile.h"

// EEPROM data ---------------------------------------------------------------------------------------
#define EE_BASE 0x10
uint16_t ee_display_timeout EEMEM = EE_BASE;			// display timeout value
uint8_t ee_display_light_level EEMEM = EE_BASE + 2;		// display back light level
uint16_t ee_batt_threshold_high EEMEM = EE_BASE + 4;	// battery voltage above this value sets high status
uint16_t ee_batt_threshold_crit EEMEM = EE_BASE + 6;	// battery voltage below this value sets critical status
uint16_t ee_batt_threshold_low EEMEM = EE_BASE + 8;		// battery voltage below this value sets low status
uint16_t ee_amb_light_threshold EEMEM = EE_BASE + 10;	// light status
uint8_t ee_amb_light_mov_avg_dur EEMEM = EE_BASE + 12;
uint8_t ee_led_tw_on_hour EEMEM = EE_BASE + 13;
uint8_t ee_led_tw_on_min EEMEM = EE_BASE + 14;
uint8_t ee_led_tw_off_hour EEMEM = EE_BASE + 15;
uint8_t ee_led_tw_off_min EEMEM = EE_BASE + 16;
uint16_t ee_led_duration EEMEM = EE_BASE + 17;
uint8_t ee_led_max EEMEM = EE_BASE + 19;
uint8_t ee_mp3_tw1_on_hour EEMEM = EE_BASE + 20;
uint8_t ee_mp3_tw1_on_min EEMEM = EE_BASE + 21;
uint8_t ee_mp3_tw1_off_hour EEMEM = EE_BASE + 22;
uint8_t ee_mp3_tw1_off_min EEMEM = EE_BASE + 23;
uint8_t ee_mp3_tw2_on_hour EEMEM = EE_BASE + 24;
uint8_t ee_mp3_tw2_on_min EEMEM = EE_BASE + 25;
uint8_t ee_mp3_tw2_off_hour EEMEM = EE_BASE + 26;
uint8_t ee_mp3_tw2_off_min EEMEM = EE_BASE + 27;
uint8_t ee_mp3_duration EEMEM = EE_BASE + 28;
uint8_t ee_hatch_hour EEMEM = EE_BASE + 29;
uint8_t ee_hatch_min EEMEM = EE_BASE + 30;
uint16_t ee_hatch_dur EEMEM = EE_BASE + 31;


/* ee_read_batt_status -------------------------------------------------------------------
* Reads the fixed parameters for battery status from eeprom
*/
void ee_read_batt_status(void) {
	extern batt_status_t batt_status;
	
	batt_status.threshold_high = eeprom_read_word(&ee_batt_threshold_high);
	if (batt_status.threshold_high > 1500) {
		batt_status.threshold_high = DEFAULT_BATT_THESHOLD_HIGH;
		eeprom_write_word(&ee_batt_threshold_high, batt_status.threshold_high);
	};
	batt_status.threshold_crit = eeprom_read_word(&ee_batt_threshold_crit);
	if (batt_status.threshold_crit > 1400) {
		batt_status.threshold_crit = DEFAULT_BATT_THESHOLD_CRIT;
		eeprom_write_word(&ee_batt_threshold_crit, batt_status.threshold_crit);
	};
	batt_status.threshold_low = eeprom_read_word(&ee_batt_threshold_low);
	if (batt_status.threshold_low > 1300) {
		batt_status.threshold_low = DEFAULT_BATT_THESHOLD_LOW;
		eeprom_write_word(&ee_batt_threshold_low, batt_status.threshold_low);
	};
};

/* ee_read_amb_light_status -------------------------------------------------------------------
* Reads the fixed parameters for light status from eeprom
*/
void ee_read_amb_light_status(void) {
	extern amb_light_status_t amb_light_status;
	
	amb_light_status.light_threshold = eeprom_read_word(&ee_amb_light_threshold);
	if (amb_light_status.light_threshold > 999) {
		amb_light_status.light_threshold  = DEFAULT_AMB_LIGHT_THRESHOLD;
		eeprom_write_word(&ee_amb_light_threshold, amb_light_status.light_threshold);
	};
	amb_light_status.light_mov_avg_dur = eeprom_read_byte(&ee_amb_light_mov_avg_dur);
	if (amb_light_status.light_mov_avg_dur > MAX_AMB_LIGHT_MOV_AVG_DUR) {
		amb_light_status.light_mov_avg_dur = DEFAULT_AMB_LIGHT_MOV_AVG_DUR;
		eeprom_write_byte(&ee_amb_light_mov_avg_dur, amb_light_status.light_mov_avg_dur);
	};
};

/* ee_read_display_status -----------------------------------------------------------------
* Reads the fixed parameters for display status from eeprom
*/
void ee_read_display_status(void) {
	extern display_status_t display_status ;
	
	display_status.timeout = eeprom_read_word(&ee_display_timeout);
	if (display_status.timeout > 1800) {
		display_status.timeout = DEFAULT_DISPLAY_TIMEOUT;
		eeprom_write_word(&ee_display_timeout, display_status.timeout);
	};
	display_status.light_level = eeprom_read_byte(&ee_display_light_level);
	if (display_status.light_level > MAX_DISPLAY_LIGHT) {
		display_status.light_level = DEFAULT_DISPLAY_LIGHT_LEVEL;
		eeprom_write_byte(&ee_display_light_level, display_status.light_level);
	};
};

/* ee_write_display_status -----------------------------------------------------------------
* Writes the fixed parameters for display status from eeprom
*/
void ee_write_display_status(void) {
	extern display_status_t display_status ;
	
	eeprom_write_word(&ee_display_timeout, display_status.timeout);
	eeprom_write_byte(&ee_display_light_level, display_status.light_level);
};

/* ee_read_LED_status ----------------------------------------------------------------------
* Reads fixed parameters for LED status from eeprom
*/
void ee_read_LED_status(void) {
	extern LED_status_t LED_status;
	
	LED_status.led_tw_on_hour = eeprom_read_byte(&ee_led_tw_on_hour);
	if (LED_status.led_tw_on_hour > 23) {
		LED_status.led_tw_on_hour = DEFAULT_LED_TW_ON_HOUR;
		eeprom_write_byte(&ee_led_tw_on_hour, LED_status.led_tw_on_hour);
	};
	LED_status.led_tw_on_min = eeprom_read_byte(&ee_led_tw_on_min);
	if (LED_status.led_tw_on_min > 59) {
		LED_status.led_tw_on_min = DEFAULT_LED_TW_ON_MIN;
		eeprom_write_byte(&ee_led_tw_on_min, LED_status.led_tw_on_min);
	};
	LED_status.led_tw_off_hour = eeprom_read_byte(&ee_led_tw_off_hour);
	if (LED_status.led_tw_off_hour > 23) {
		LED_status.led_tw_off_hour = DEFAULT_LED_TW_OFF_HOUR;
		eeprom_write_byte(&ee_led_tw_off_hour, LED_status.led_tw_off_hour);
	};
	LED_status.led_tw_off_min = eeprom_read_byte(&ee_led_tw_off_min);
	if (LED_status.led_tw_off_min > 59) {
		LED_status.led_tw_off_min = DEFAULT_LED_TW_OFF_MIN;
		eeprom_write_byte(&ee_led_tw_off_min, LED_status.led_tw_off_min);
	};
	LED_status.led_duration = eeprom_read_word(&ee_led_duration);
	if (LED_status.led_duration > 6000) {
		LED_status.led_duration = DEFAULT_LED_DURATION;
		eeprom_write_word(&ee_led_duration, LED_status.led_duration);
	};
	LED_status.led_max = eeprom_read_byte(&ee_led_max);
	if (LED_status.led_max > MAX_LED_LEVEL) {
		LED_status.led_max = DEFAULT_LED_MAX;
		eeprom_write_byte(&ee_led_max, LED_status.led_max);
	};
}

/* ee_write_LED_status ----------------------------------------------------------------------
* Writes the fixed parameters for LED status to eeprom
*/
void ee_write_LED_status(void) {
	extern LED_status_t LED_status;
	
	eeprom_write_byte(&ee_led_tw_on_hour, LED_status.led_tw_on_hour);
	eeprom_write_byte(&ee_led_tw_on_min, LED_status.led_tw_on_min);
	eeprom_write_byte(&ee_led_tw_off_hour, LED_status.led_tw_off_hour);
	eeprom_write_byte(&ee_led_tw_off_min, LED_status.led_tw_off_min);
	eeprom_write_word(&ee_led_duration, LED_status.led_duration);
	eeprom_write_byte(&ee_led_max, LED_status.led_max);
}


/* ee_read_mp3_status ----------------------------------------------------------------------
* Reads fixed parameters for mp3 status from eeprom
*/
void ee_read_mp3_status(void) {
	extern mp3_status_t mp3_status;
	
	// timw window 1
	mp3_status.mp3_tw1_on_hour = eeprom_read_byte(&ee_mp3_tw1_on_hour);
	if (mp3_status.mp3_tw1_on_hour > 23) {
		mp3_status.mp3_tw1_on_hour = DEFAULT_MP3_TW1_ON_HOUR;
		eeprom_write_byte(&ee_mp3_tw1_on_hour, mp3_status.mp3_tw1_on_hour);
	};
	mp3_status.mp3_tw1_on_min = eeprom_read_byte(&ee_mp3_tw1_on_min);
	if (mp3_status.mp3_tw1_on_min > 59) {
		mp3_status.mp3_tw1_on_min = DEFAULT_MP3_TW1_ON_MIN;
		eeprom_write_byte(&ee_mp3_tw1_on_min, mp3_status.mp3_tw1_on_min);
	};
	mp3_status.mp3_tw1_off_hour = eeprom_read_byte(&ee_mp3_tw1_off_hour);
	if (mp3_status.mp3_tw1_off_hour > 23) {
		mp3_status.mp3_tw1_off_hour = DEFAULT_MP3_TW1_OFF_HOUR;
		eeprom_write_byte(&ee_mp3_tw1_off_hour, mp3_status.mp3_tw1_off_hour);
	};
	mp3_status.mp3_tw1_off_min = eeprom_read_byte(&ee_mp3_tw1_off_min);
	if (mp3_status.mp3_tw1_off_min > 59) {
		mp3_status.mp3_tw1_off_min = DEFAULT_MP3_TW1_OFF_MIN;
		eeprom_write_byte(&ee_mp3_tw1_off_min, mp3_status.mp3_tw1_off_min);
	};
	
	// time window 2
	mp3_status.mp3_tw2_on_hour = eeprom_read_byte(&ee_mp3_tw2_on_hour);
	if (mp3_status.mp3_tw2_on_hour > 23) {
		mp3_status.mp3_tw2_on_hour = DEFAULT_MP3_TW2_ON_HOUR;
		eeprom_write_byte(&ee_mp3_tw1_on_hour, mp3_status.mp3_tw2_on_hour);
	};
	mp3_status.mp3_tw2_on_min = eeprom_read_byte(&ee_mp3_tw2_on_min);
	if (mp3_status.mp3_tw2_on_min > 59) {
		mp3_status.mp3_tw2_on_min = DEFAULT_MP3_TW2_ON_MIN;
		eeprom_write_byte(&ee_mp3_tw2_on_min, mp3_status.mp3_tw2_on_min);
	};
	mp3_status.mp3_tw2_off_hour = eeprom_read_byte(&ee_mp3_tw2_off_hour);
	if (mp3_status.mp3_tw2_off_hour > 23) {
		mp3_status.mp3_tw2_off_hour = DEFAULT_MP3_TW2_OFF_HOUR;
		eeprom_write_byte(&ee_mp3_tw2_off_hour, mp3_status.mp3_tw2_off_hour);
	};
	mp3_status.mp3_tw2_off_min = eeprom_read_byte(&ee_mp3_tw2_off_min);
	if (mp3_status.mp3_tw2_off_min > 59) {
		mp3_status.mp3_tw2_off_min = DEFAULT_MP3_TW2_OFF_MIN;
		eeprom_write_byte(&ee_mp3_tw2_off_min, mp3_status.mp3_tw2_off_min);
	};
	
	// duration
	mp3_status.mp3_duration = eeprom_read_byte(&ee_mp3_duration);
	if (mp3_status.mp3_duration > 30) {
		mp3_status.mp3_duration = DEFAULT_MP3_DURATION;
		eeprom_write_byte(&ee_mp3_duration, mp3_status.mp3_duration);
	};
}

/* ee_read_hatch_control -------------------------------------------------------------
* Reads fixed parameters for hatch control from eeprom
*/
void ee_read_hatch_control(void) {
	extern hatch_control_t hatch_control;

	hatch_control.hatch_hour = eeprom_read_byte(&ee_hatch_hour);
	if (hatch_control.hatch_hour > 23) {
		hatch_control.hatch_hour = DEFAULT_HATCH_HOUR;
		eeprom_write_byte(&ee_hatch_hour, hatch_control.hatch_hour);
	};
	hatch_control.hatch_min = eeprom_read_byte(&ee_hatch_min);
	if (hatch_control.hatch_min > 59) {
		hatch_control.hatch_min = DEFAULT_HATCH_MIN;
		eeprom_write_byte(&ee_hatch_min, hatch_control.hatch_min);
	};
	hatch_control.hatch_dur = eeprom_read_word(&ee_hatch_dur);
	if (hatch_control.hatch_dur > 180) {
		hatch_control.hatch_dur = DEFAULT_HATCH_DUR;
		eeprom_write_word(&ee_hatch_dur, hatch_control.hatch_dur);
	};
}

/* ee_write_hatch_control ------------------------------------------------------------
* Writes hatch control parameters to eeprom
*/
void ee_write_hatch_control(void) {
	extern hatch_control_t hatch_control;

	eeprom_write_byte(&ee_hatch_hour, hatch_control.hatch_hour);
	eeprom_write_byte(&ee_hatch_min, hatch_control.hatch_min);
	eeprom_write_word(&ee_hatch_dur, hatch_control.hatch_dur);
}

/* ee_write_amb_light_status -------------------------------------------------------------------
* Reads the fixed parameters for light status from eeprom
*/
void ee_write_amb_light_status(void) {
	extern amb_light_status_t amb_light_status;
	
	eeprom_write_word(&ee_amb_light_threshold, amb_light_status.light_threshold);
	eeprom_write_byte(&ee_amb_light_mov_avg_dur, amb_light_status.light_mov_avg_dur);
};
